﻿$(function() {

    (function($) {
        "use strict";

        var configMap = {
            maritalStatusDs: null,
            maritalStatus: [],
            employeesDs: null,
            employees: [],
            employeeModel: null,
            referenceModel: null,
            $form: "#employee-form",
            $grid: "#grid",
            $head: ".section-head",
            $content: ".section-content"
        };

        function getUid() {
            return $(configMap.$form).data("uid");
        }

        function setUid(value) {
            $(configMap.$form).data("uid", value);
        }

        function getKendoGrid() {
            return $(configMap.$grid).data("kendoGrid");
        }

        function displayForm(flag) {
            if (flag) {
                $("#employee-form").show();
            } else {
                $("#employee-form").hide();
            }
        }

        function reset(clearSelection) {
            clearSelection = typeof clearSelection !== "undefined" ? clearSelection : true;

            $(configMap.$form).find("input").val("");

            var numerictextboxes = $(configMap.$form).find("input[data-role='numerictextbox']");
            $.each(numerictextboxes, function (idx, numericbox) {
                $(numericbox).data("kendoNumericTextBox").value("");
            });

            var datepickers = $(configMap.$form).find("input[data-role='datepicker']");
            $.each(datepickers, function (idx, datepicker) {
                $(datepicker).data("kendoDatePicker").value("");
            });

            var dropdownlists = $(configMap.$form).find("select[data-role='dropdownlist']");
            $.each(dropdownlists, function (idx, dropdownlist) {
                $(dropdownlist).data("kendoDropDownList").value("");
            });

            setUid("");

            displayForm(!clearSelection);

            if (clearSelection) {
                getKendoGrid().clearSelection();
            }
        }

        function configureDataSources() {
            configMap.ReferenceModel = kendo.data.Model.define({
                id: "Id",
                fields: {
                    Id: { editable: false, defaultValue: 0 },
                    Code: { type: "string" }
                }
            });

            configMap.EmployeeModel = kendo.data.Model.define({
                id: "EmployeeId",
                fields: {
                    EmployeeId: { editable: false, defaultValue: 0 },
                    FirstName: { type: "string" },
                    LastName: { type: "string" },
                    MaritalStatus: { defaultValue: {} },
                    JobTitle: { type: "string" },
                    HireDate: { type: "date" },
                    BirthDate: { type: "date" },
                    VacationDays: { type: "number" },
                    SickLeaveDays: { type: "number" },
                    Salary: { type: "number" }
                }
            });

            configMap.maritalStatusDs = new kendo.data.DataSource({
                transport: {
                    read: function(options) {
                        options.success(configMap.maritalStatus);
                    }
                },
                schema: {
                    model: configMap.ReferenceModel
                }
            });

            configMap.employeesDs = new kendo.data.DataSource({
                transport: {
                    read: function(options) {
                        options.success(configMap.employees);

                        // update cid
                        $.each(configMap.employeesDs.data(), function(idx, record) {
                            if (!record.hasOwnProperty("cid") || !record["cid"]) { // not present or null
                                record["cid"] = record["uid"];
                            }
                        });
                    },
                    create: function(options) {
                        $.ajax({
                            cache: false,
                            type: "POST",
                            url: "/api/Employees/Save",
                            contentType: "application/json; charset=utf-8",
                            dataType: "json",
                            data: JSON.stringify( options.data ),
                            success: function(result) {
                                console.log("employees saved successfully");
                                options.success(result);
                            },
                            error: function(result) {
                                options.error(result);
                            }
                        });
                    },
                    update: function(options) {
                        $.ajax({
                            cache: false,
                            type: "POST",
                            url: "/api/Employees/Update",
                            contentType: "application/json; charset=utf-8",
                            dataType: "json",
                            data: JSON.stringify( options.data ),
                            success: function(result) {
                               console.log("employees updated successfully");
                               options.success(result);
                            },
                            error: function(result) {
                                // when datatype is "json", this event fires 
                                // only if response cannot be converted to json                        
                                options.error(result);
                            }
                        });
                    }
                },
                schema: {
                    model: configMap.EmployeeModel,
                    errors: "errors"
                }
            });

            function onSuccessRead(data) {

                if (data.Employees) {
                    configMap.employees = $.map(data.Employees, function (record, idx) {
                        return record;
                    });
                    configMap.employeesDs.read();
                }

                if (data.MaritalStatus) {
                    configMap.maritalStatus = $.map(data.MaritalStatus, function (record, idx) {
                        return record;
                    });
                    configMap.maritalStatusDs.read();
                }
            }

            function onErrorRead(xhr, textStatus, errorThrown) {
                console.log(errorThrown);
            }
   
            // read
            $.ajax({
                type: "GET",
                cache: false,
                url: "/api/Employees/",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                success: onSuccessRead,
                error: onErrorRead
            });
        }

        function configureForm() {
            var $container = $(configMap.$form);

            $container.find("select[id='MaritalStatus']").kendoDropDownList({
                dataTextField: "Code",
                dataValueField: "Id",
                dataSource: configMap.maritalStatusDs,
                optionLabel: { Code: "Select Marital Status", Id: -1 }
            });

            $container.find("input[id='HireDate'], input[id='BirthDate']").kendoDatePicker({
                format: "dd MMM yyyy"
            }).attr("readonly", true);

            $container.find("input[id='VacationDays'], input[id='SickLeaveDays']").kendoNumericTextBox({
                min: 0,
                max: 100,
                step: 1,
                decimals: 0,
                format: "#"
            });

            $container.find("input[id='Salary']").kendoNumericTextBox({
                min: 0,
                max: 1000000000,
                step: 500
            });
        }

        function populateEmployeeForm(data) {
            if (!data) return;

            $(configMap.$form)
                .find("input[id='FirstName']").val(data.FirstName).end()
                .find("input[id='LastName']").val(data.LastName).end()
                .find("input[id='JobTitle']").val(data.JobTitle).end();

            $(configMap.$form).find("input[id='BirthDate']").data("kendoDatePicker").value(data.BirthDate);
            $(configMap.$form).find("input[id='HireDate']").data("kendoDatePicker").value(data.HireDate);

            var maritalStatus = data.MaritalStatus;
            if (maritalStatus) {
                $(configMap.$form).find("select[id='MaritalStatus']").data("kendoDropDownList").value(data.MaritalStatus.Id);
            }
            
            $(configMap.$form).find("input[id='VacationDays']").data("kendoNumericTextBox").value(data.VacationDays);
            $(configMap.$form).find("input[id='SickLeaveDays']").data("kendoNumericTextBox").value(data.SickLeaveDays);
            $(configMap.$form).find("input[id='Salary']").data("kendoNumericTextBox").value(data.Salary);
        }

        function showEmployeeDetails(data) {
            if (!data) return;

            reset(false); // reset first

            setUid(data.uid); // store ui on form

            populateEmployeeForm(data); // populate form with employee data
        }

        function configureGrid() {
            function onGridChange(e) {
                var grid = e.sender;
                var dataItem = grid.dataItem(grid.select()); // get selected row data item

                if (!dataItem) return;

                showEmployeeDetails(dataItem);
            }

            $(configMap.$grid).kendoGrid({
                dataSource: configMap.employeesDs,
                change: onGridChange,
                selectable: "single",
                navigatable: true,
                sortable: true,
                filterable: { extra: false },
                columns: [
                    { field: "EmployeeId", title: "ID", width: "75px" },
                    { field: "FirstName", title: "First Name", width: "150px" },
                    { field: "LastName", title: "Last Name", width: "150px" },
                    { field: "JobTitle", title: "Job Title", width: "200px" },
                    { field: "BirthDate", title: "Birth Date", width: "150px", template: "#= getBirthDate(data) #" }, // universal date format
                    { field: "HireDate", title: "Hire Date", width: "150px", template: "#= getHireDate(data) #" },
                    { field: "MaritalStatus", title: "Marital Status", width: "150px", template: "#= getMaritalStatusCode(data) #" },
                    { field: "VacationDays", title: "Vacation", width: "150px" },
                    { field: "SickLeaveDays", title: "Sick Leave", width: "150px" },
                    { field: "Salary", title: "Salary ($)", width: "150px", format: "{0:n}" }
                ]
            });
        }

        function handleAddNewRecord(e) {
            reset(true);
            displayForm(true);
        }

        function handleCancelChanges(e) {
            var currentUid = getUid(); // get uid for currently displayed row

            if (currentUid) {
                var curentDataItem = configMap.employeesDs.getByUid(currentUid); // get current data item
                configMap.employeesDs.cancelChanges(curentDataItem); // cancel changes made on the current data item
            }

            reset(true);
        }

        function handleSaveChanges(e) {
            function updateEmployee(uid) {
                function getMaritalStatusItem(id) {
                    var tempDs = new kendo.data.DataSource({
                        data: configMap.maritalStatusDs.data(),
                        filter:
                        [
                            { field: "Id", operator: "eq", value: id }
                        ]
                    });
                    tempDs.read();

                    var result = null;
                    if (tempDs.view().length > 0) {
                        result = tempDs.view()[0];
                    }

                    return result;
                }
                
                function makeEmployeeModel() {
                    var model = new configMap.EmployeeModel({
                        EmployeeId: 0,
                        MaritalStatus: new configMap.ReferenceModel({})
                    });

                    return model;
                }

                function addEmployeeToDataSource() {
                    var model = makeEmployeeModel();

                    configMap.employeesDs.add(model);

                    if (!model.hasOwnProperty("cid") || !model["cid"]) { // not present or null
                        model["cid"] = model["uid"];
                    }

                    return model;
                }

                var model;
                if (uid) {
                    model = configMap.employeesDs.getByUid(uid); // get current data item
                }

                if (typeof model === "undefined" || model === null) { // new record
                    model = addEmployeeToDataSource();
                }

                if (typeof model === "undefined" || model === null) return;

                model.set("FirstName", $(configMap.$form).find("input[id='FirstName']").val());
                model.set("LastName", $(configMap.$form).find("input[id='LastName']").val());
                model.set("JobTitle", $(configMap.$form).find("input[id='JobTitle']").val());

                var birthDate = model.BirthDate;
                var birthDateObject = $("input[id='BirthDate']").data("kendoDatePicker");
                if (birthDateObject.value()) {
                    birthDate = kendo.parseDate(birthDateObject.value());
                }
                model.set("BirthDate", birthDate);

                var hireDate = model.HireDate;
                var hireDateObject = $("input[id='HireDate']").data("kendoDatePicker");
                if (hireDateObject.value()) {
                    hireDate = kendo.parseDate(hireDateObject.value());
                }
                model.set("HireDate", hireDate);

                var maritalStatus = model.maritalStatus;
                var maritalStatusObject = $("select[id='MaritalStatus']").data("kendoDropDownList");
                if (maritalStatusObject.value()) {
                    maritalStatus = kendo.parseInt(maritalStatusObject.value());
                }
                model.set("MaritalStatus", getMaritalStatusItem(maritalStatus));

                var vacation = model.VacationDays;
                var vacationObject = $("input[id='VacationDays']").data("kendoNumericTextBox");
                if (vacationObject.value()) {
                    vacation = kendo.parseInt(vacationObject.value());
                }
                model.set("VacationDays", vacation);

                var sickLeave = model.SickLeaveDays;
                var sickLeaveObject = $("input[id='SickLeaveDays']").data("kendoNumericTextBox");
                if (sickLeaveObject.value()) {
                    sickLeave = kendo.parseInt(sickLeaveObject.value());
                }
                model.set("SickLeaveDays", sickLeave);

                var salary = model.Salary;
                var salaryObject = $("input[id='Salary']").data("kendoNumericTextBox");
                if (salaryObject.value()) {
                    salary = kendo.parseFloat(salaryObject.value());
                }
                model.set("Salary", salary);
            }

            var currentUid = getUid(); // get uid for currently displayed row

            updateEmployee(currentUid); // update employee record in DataSource

            reset(true);

            getKendoGrid().saveChanges(); // save changes to server using DataSource
        }

        function handleToggle(e, me) {
            e.preventDefault();
            e.stopPropagation();

            var iconSpan = $(me).find("span:nth-child(1)");
            var textSpan = $(me).find("span:nth-child(2)");

            getKendoGrid().wrapper.toggle(); // toggle grid

            $(configMap.$content).find("input[id='Filter']").toggle(); // toggle filter
            
            $(iconSpan).toggleClass("glyphicon-eye-close glyphicon-eye-open"); // change icons

            // show/hide grid message
            if ($(textSpan).text() === "Hide Grid")
                $(textSpan).text("Show Grid");
            else
                $(textSpan).text("Hide Grid");
        }

        function handleFilterChange(e) {

            var value = $(e.target).val();
            if (value.length > 0) { // there is text
                getKendoGrid().dataSource.filter({
                    logic: "or",
                    filters: [
                        { field: "FirstName", operator: "contains", value: value },
                        { field: "LastName", operator: "contains", value: value }
                    ]
                });
            } else {
                getKendoGrid().dataSource.filter({});
            }
        }

        function configureEventHandling(e) {
            $(".section-content").on("click", "button.add-action", function (e) {
                e.preventDefault();
                handleAddNewRecord(e);
            });

            $(".section-content").on("keyup click input", "input[id='Filter']", function (e) {
                e.preventDefault();
                handleFilterChange(e);
            });

            $("#employee-form").on("click", "button.emp-save", function (e) {
                e.preventDefault();
                handleSaveChanges(e);
            });

            $("#employee-form").on("click", "button.emp-cancel", function (e) {
                e.preventDefault();
                handleCancelChanges(e);
            });

            $(".section-content").on("click", "button.toggle", function (e) {
                e.preventDefault();
                handleToggle(e, $(this));
            });
        }

        configureDataSources();
        configureForm();
        configureGrid();
        configureEventHandling();

    })(jQuery);

});